// Communicator.h
//
// Copyright (C) 1997 Ficus Kirkpatrick
//

#pragma once

// Message constants.
//
enum
{
	F_NET_CONNECT		= 'fcon',
	F_NET_CONNECTED		= 'fcnd',
	F_NET_LISTEN		= 'flsn',
	F_NET_SEND			= 'fsnd',
	F_NET_RECEIVED		= 'frcv',
	F_NET_BYE			= 'fbye',
	F_NET_FATAL			= 'fftl'
};

// Default buffer size.
//
const int F_NET_BUFSIZ = 1024;

class FCommunicator : public BLooper
{
// The constructor and destructor.
public:
	FCommunicator(BHandler *, int = F_NET_BUFSIZ);
	virtual ~FCommunicator();
	
// Virtual functions.
public:
	virtual void MessageReceived(BMessage *);

// Functions for external use.
public:
	void Connect(const char *, int);
	void Listen(int);
	void Send(const char *, int);
	
// Functions for internal use.
private:
	void RelayMessage(BMessage *);

// Thread entry functions.
private:
	static long NetListener(void *);
	static long NetConnecter(void *);
	static long NetReader(void *);
	static long NetWriter(void *);

// Undocumented voodoo function.
private:
	static void ThreadSuicide(int);
		
// Private data.
private:
	BHandler *fHandler;     // the message handler
	int fControl;           // server control socket
	int fSocket;            // client socket

	int fBufferSize;        // size of the buffers
	char *fInBuffer;        // received data buffer
	char *fOutBuffer;       // outbound data buffer
	
	char *fLocalHost;       // local host
	int fLocalPort;         // local port
	char *fRemoteHost;      // remote host
	int fRemotePort;        // remote port
	
	thread_id fInitiator;   // connect/listen thread id
	thread_id fReader;      // reader thread id
	thread_id fWriter;      // writer thread id

	port_id fSendPort;      // outbound data port

	bool fQuit;				// for notifying child threads
	bool fConnected;		// are we connected yet?

};

