// FCTester.cpp
//
// A simple example application that makes use of
// the Communicator library.
//
// No project file has been included for this; if
// you want to run it, just compile it from a
// Terminal window like so:
//
// mwcc -I.. -c FCTester.cpp
// mwld -o FCTester FCTester.o ../Communicator.lib
//

#include <stdio.h>
#include <unistd.h>
#include <AppKit.h>

#include "Communicator.h"

class FApplication : public BApplication
{
public:
	FApplication();
	virtual ~FApplication();
	virtual void MessageReceived(BMessage *);

private:
	FCommunicator *fNetwork;

};


// main
//
// This basically does what almost every other main()
// you'll see on the BeOS does. :-)
//
int main(void)
{
	FApplication *app;
	
	app = new FApplication();
	app->Run();
	delete app;
	
	return(0);
}

// FApplication::FApplication
//
// Boring constructor;  instantiates the FCommunicator
// and listens for connections on port 3333.
//
FApplication::FApplication()
             : BApplication('????')
{
	fNetwork = new FCommunicator(this);
	fNetwork->Listen(3333);
}

// FApplication::~FApplication
//
// An even more boring destructor.  Gets rid of the
// FCommunicator object (makes sure all of its threads
// die, buffers are cleaned up, etc.)
//
FApplication::~FApplication()
{
	if (fNetwork) { fNetwork->Quit(); }
}

// FApplication::MessageReceived
//
// Finally, the least bland of all.  This handles
// the network messages that are passed to us.
//
void FApplication::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
	case F_NET_CONNECTED:
		printf("A client has connected.\n");
		printf("-----------------------\n");
		fflush(stdout);
		break;
	
	case F_NET_RECEIVED:
		long len;
		void *ptr = msg->FindData("data", B_RAW_TYPE, &len);
		write(1, ptr, len);
		break;
	
	case F_NET_BYE:
		printf("\n-------------------------------\n");
		printf("The connection has been closed.\n");
		fflush(stdout);
		Quit();
		break;
		
	case F_NET_FATAL:
		printf("\n--------------------------------\n");
		printf("There was a fatal network error.\n");
		fflush(stdout);
		Quit();
		break;

	default:
		inherited::MessageReceived(msg);
		break;
	}
}

